# This file was auto-generated by Fern from our API Definition.

from typing import Any, Dict, Mapping, Optional


def remove_none_from_dict(original: Mapping[str, Optional[Any]]) -> Dict[str, Any]:
    new: Dict[str, Any] = {}
    for key, value in original.items():
        if value is not None:
            new[key] = value
    return new



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1


class TopLogprobs(pydantic_v1.BaseModel):
    """
    An OpenAI API compatible schema for logprobs output.
    """

    bytes: typing.Optional[typing.List[typing.Any]] = pydantic_v1.Field(default=None)
    """
    A list of integers representing the UTF-8 bytes representation of the token. Useful in instances where characters are represented by multiple tokens and their byte representations must be combined to generate the correct text representation. Can be null if there is no bytes representation for the token.
    """

    logprob: float = pydantic_v1.Field()
    """
    Logprob corresponding to the top token.
    """

    token: str = pydantic_v1.Field()
    """
    Token from the top list.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



import json
import logging
import time
from octoai.uploading_asset_library import AsyncUploadingAssetLibraryClient, UploadingAssetLibraryClient
from .base_client import BaseOctoAI, AsyncBaseOctoAI

import os
import typing
import httpx

from .asset_library.client import AssetLibraryClient, AsyncAssetLibraryClient
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .core.pydantic_utilities import pydantic_v1
from .environment import OctoAIEnvironment
from .fine_tuning.client import AsyncFineTuningClient, FineTuningClient
from .image_gen.client import AsyncImageGenClient, ImageGenClient
from .text_gen.client import AsyncTextGenClient, TextGenClient
from .util import retry


_LOG = logging.getLogger(__name__)

DEFAULT_HEALTH_CHECK_TIMEOUT_SECONDS = 900.0

class _JSONEncoder(json.JSONEncoder):
    def default(self, o: typing.Any) -> typing.Any:
        if isinstance(o, bytes):
            return o.decode()
        return json.JSONEncoder.default(self, o)


class InferenceFuture(pydantic_v1.BaseModel):
    """Response class for endpoints that support server side async inferences.

    :param response_id: Unique identifier for inference
    :type response_id: str
    :param poll_url: URL to poll status of inference.
    :type poll_url: str
    """

    response_id: str
    poll_url: str


class OctoAI(BaseOctoAI):
    asset_library: UploadingAssetLibraryClient

    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - environment: OctoAIEnvironment. The environment to use for requests from the client. from .environment import OctoAIEnvironment

                                          Defaults to OctoAIEnvironment.PRODUCTION

        - api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]].

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds.

        - httpx_client: typing.Optional[httpx.Client]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from octoai.client import OctoAI

    client = OctoAI(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        environment: OctoAIEnvironment = OctoAIEnvironment.PRODUCTION,
        api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("OCTOAI_TOKEN"),
        timeout: typing.Optional[float] = 60,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        super().__init__(
            environment=environment,
            api_key=api_key,
            timeout=timeout,
            httpx_client=httpx_client
        )

        self._httpx_client = self._client_wrapper.httpx_client
        self.asset_library = UploadingAssetLibraryClient(client_wrapper=self._client_wrapper)

    def _error(self, resp: httpx.Response):
        resp.raise_for_status()

    def infer(self, endpoint_url: str, inputs: typing.Mapping[str, typing.Any]) -> typing.Mapping[str, typing.Any]:
        """Send a request to the given endpoint URL with inputs as request body.

        :param endpoint_url: target endpoint
        :type endpoint_url: str
        :param inputs: inputs for target endpoint
        :type inputs: Mapping[str, Any]

        :raises OctoAIServerError: server-side failures (unreachable, etc)
        :raises OctoAIClientError: client-side failures (throttling, unset token)

        :return: outputs from endpoint
        :rtype: Mapping[str, Any]
        """
        resp = retry(
            lambda: self._httpx_client.httpx_client.post(
                url=endpoint_url,
                headers={"Content-Type": "application/json"},
                content=json.dumps(inputs, cls=_JSONEncoder),
            )
        )
        if resp.status_code != 200:
            self._error(resp)
        return resp.json()

    def infer_async(
        self, endpoint_url: str, inputs: typing.Mapping[str, typing.Any]
    ) -> InferenceFuture:
        """Execute an inference in the background on the server.

        :class:`InferenceFuture` allows you to query status and get results
        once it's ready.

        :param endpoint_url: url to post inference request
        :type endpoint_url: str
        :param inputs: inputs to send to endpoint
        :type inputs: Mapping[str, Any]
        """
        resp = retry(
            lambda: self._httpx_client.httpx_client.post(
                url=endpoint_url, json=inputs, headers={"X-OctoAI-Async": "1"}
            )
        )
        if resp.status_code >= 400:
            self._error(resp)
        resp_json = resp.json()
        future = InferenceFuture(**resp_json)
        return future

    def infer_stream(
        self, endpoint_url: str, inputs: typing.Mapping[str, typing.Any], map_fn: typing.Optional[typing.Callable] = None
    ) -> typing.Iterator[dict]:
        """Stream text event response body for supporting endpoints.

        This is an alternative to loading all response body into memory at once.

        :param endpoint_url: target endpoint
        :type endpoint_url: str
        :param inputs: inputs for target endpoint such as a prompt and other parameters
        :type inputs: Mapping[str, Any]
        :param inputs: function to map over each response
        :type inputs: Callable
        :return: Yields a :class:`dict` that contains the server response data.
        :rtype: Iterator[:class:`dict`]
        """
        with self._httpx_client.httpx_client.stream(
            method="POST",
            url=endpoint_url,
            content=json.dumps(inputs, cls=_JSONEncoder),
            headers={"accept": "text/event-stream"},
        ) as resp:
            if resp.status_code >= 400:
                # Loads response body on error for streaming responses
                resp.read()
                self._error(resp)

            for payload in resp.iter_lines():
                # Empty lines used to separate payloads.
                if payload == "":
                    continue
                # End of stream (OpenAPI /v1/chat/completions).
                elif payload == "data: [DONE]":
                    break
                # Event data identified with "data:"  JSON inside.
                elif payload.startswith("data:"):
                    payload_dict = json.loads(payload.lstrip("data:"))
                    if map_fn:
                        yield map_fn(payload_dict)
                    else:
                        yield payload_dict
                # Any other input is a malformed response.
                else:
                    raise ValueError(
                        f"Stream response is malformed: {payload}"
                    )


    def _poll_future(self, future: InferenceFuture) -> typing.Dict[str, str]:
        """Get from poll_url and return response.

        :param future: Future from :meth:`OctoAI.infer_async`
        :type future: :class:`InferenceFuture`
        :returns: Dictionary with response
        :rtype: Dict[str, str]
        """
        response = self._httpx_client.httpx_client.get(url=future.poll_url)
        if response.status_code >= 400:
            self._error(response)
        return response.json()

    def is_future_ready(self, future: InferenceFuture) -> bool:
        """Return whether the future's result has been computed.

        This class will raise any errors if the status code is >= 400.

        :param future: Future from :meth:`OctoAI.infer_async`
        :type future: :class:`InferenceFuture`
        :returns: True if able to use :meth:`OctoAI.get_future_result`
        """
        resp_dict = self._poll_future(future)
        return "completed" == resp_dict.get("status")

    def get_future_result(self, future: InferenceFuture) -> typing.Optional[typing.Dict[str, typing.Any]]:
        """Return the result of an inference.

        This class will raise any errors if the status code is >= 400.

        :param future: Future from :meth:`Client.infer_async`
        :type future: :class:`InferenceFuture`
        :returns: None if future is not ready, or dict of the response.
        :rtype: Dict[str, Any], optional
        """
        resp_dict = self._poll_future(future)
        if resp_dict.get("status") != "completed":
            return None
        response_url = resp_dict.get("response_url")
        assert response_url is not None, "response_url is missing from response"
        response = self._httpx_client.httpx_client.get(response_url)
        if response.status_code >= 400:
            self._error(response)
        return response.json()

    def health_check(
        self, endpoint_url: str, timeout: float = DEFAULT_HEALTH_CHECK_TIMEOUT_SECONDS
    ) -> int:
        """Check health of an endpoint using a get request.  Try until timeout.

        :param endpoint_url: URL as a str starting with https permitting get requests.
        :type endpoint_url: str
        :param timeout: Seconds before request times out, defaults to 900.
        :type timeout: float
        :return: status code from get request.  200 means ready.
        :rtype: int
        """
        resp = self._health_check(
            lambda: self._httpx_client.httpx_client.get(url=endpoint_url), timeout=timeout
        )
        if resp.status_code >= 400:
            self._error(resp)
        return resp.status_code

    def _health_check(
        self,
        fn: typing.Callable[[], httpx.Response],
        timeout: float = DEFAULT_HEALTH_CHECK_TIMEOUT_SECONDS,
        interval: float = 1.0,
        iteration_count: int = 0,
    ) -> httpx.Response:
        """Check the health of an endpoint.

        :param fn: Get request for endpoint health check.
        :type fn: Callable[[], httpx.Response]
        :param timeout: seconds before health_check times out, defaults to 900.0
        :type timeout: int, optional
        :param interval: seconds to wait before checking endpoint health again,
            defaults to 1.0
        :type interval: int, optional
        :param iteration_count: count total attempts for cold start warning,
            defaults to 0
        :type iteration_count: int
        :return: Response once timeout has passed
        :rtype: httpx.Response
        """
        start = time.time()
        try:
            resp = fn()
            if timeout <= 0:
                return resp
            # Raise HTTPStatusError for 4xx or 5xx.
            resp.raise_for_status()
        except httpx.HTTPStatusError:
            if 400 <= resp.status_code < 500:
                # Raise client errors. Do not retry.
                return resp
            if iteration_count == 0 and self.__class__.__name__ == "Client":
                _LOG.warning(
                    "Your endpoint may take several minutes to start and be ready to "
                    "serve inferences. You can increase your endpoint's min replicas "
                    "to mitigate cold starts."
                )
            if resp.status_code >= 500:
                stop = time.time()
                current = stop - start
                time.sleep(interval)
                return self._health_check(
                    fn, timeout - current - interval, interval, iteration_count + 1
                )
        # Raise error without retry on all other exceptions.
        return resp


class AsyncOctoAI(AsyncBaseOctoAI):
    asset_library: AsyncUploadingAssetLibraryClient

    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - environment: OctoAIEnvironment. The environment to use for requests from the client. from .environment import OctoAIEnvironment

                                          Defaults to OctoAIEnvironment.PRODUCTION

        - api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]].

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds.

        - httpx_client: typing.Optional[httpx.AsyncClient]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from octoai.client import AsyncOctoAI

    client = AsyncOctoAI(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        environment: OctoAIEnvironment = OctoAIEnvironment.PRODUCTION,
        api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("OCTOAI_TOKEN"),
        timeout: typing.Optional[float] = 60,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        super().__init__(
            environment=environment,
            api_key=api_key,
            timeout=timeout,
            httpx_client=httpx_client
        )

        self.asset_library = AsyncUploadingAssetLibraryClient(client_wrapper=self._client_wrapper)


# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .lora_tune_checkpoint import LoraTuneCheckpoint
from .lora_tune_file import LoraTuneFile


class LoraTune(pydantic_v1.BaseModel):
    """
    A LoRA tune model that includes tune configurations.
    """

    base_checkpoint: LoraTuneCheckpoint = pydantic_v1.Field()
    """
    The base checkpoint used for this LoRA Tune.
    """

    files: typing.List[LoraTuneFile] = pydantic_v1.Field()
    """
    The files used for this LoRA Tune.
    """

    resize_images: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Resize training images to the "native" dimensions of the respective engine prior to tuning. Images will be proportinally resized to such that the shorter edge will fit the native dimension. For Stable Diffusion 1.5, this is 512 pixels, and for Stable Diffusion XL, this is 1024 pixels. If the shorter edge is shorter than the native dimension, no scaling up will be performed.
    """

    seed: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    The seed used for this LoRA Tune.
    """

    steps: int = pydantic_v1.Field()
    """
    The number of steps used for this LoRA Tune.
    """

    trigger_words: typing.List[str] = pydantic_v1.Field()
    """
    The trigger words used for this tune. As of now, only one trigger word is supported. `List` is used for future-proofing.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

Status = typing.Union[
    typing.Literal["ready_to_upload", "uploaded", "ready", "deleted", "rejected", "error", "uploading"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import json
import typing
from json.decoder import JSONDecodeError

import httpx_sse

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from .errors.internal_server_error import InternalServerError
from .errors.unprocessable_entity_error import UnprocessableEntityError
from .types.chat_completion_chunk import ChatCompletionChunk
from .types.chat_completion_response import ChatCompletionResponse
from .types.chat_completion_response_format import ChatCompletionResponseFormat
from .types.chat_message import ChatMessage
from .types.completion_response import CompletionResponse
from .types.create_chat_completion_request_tool_choice import CreateChatCompletionRequestToolChoice
from .types.create_chat_completion_stream_request_tool_choice import CreateChatCompletionStreamRequestToolChoice
from .types.error_response import ErrorResponse
from .types.http_validation_error import HttpValidationError
from .types.prompt import Prompt
from .types.stop import Stop
from .types.stream_options import StreamOptions
from .types.tool_definition import ToolDefinition

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TextGenClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_chat_completion_stream(
        self,
        *,
        messages: typing.Sequence[ChatMessage],
        model: str,
        frequency_penalty: typing.Optional[float] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        loglikelihood: typing.Optional[bool] = OMIT,
        logprobs: typing.Optional[bool] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        n: typing.Optional[int] = OMIT,
        peft: typing.Optional[str] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        response_format: typing.Optional[ChatCompletionResponseFormat] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        stream_options: typing.Optional[StreamOptions] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        tool_choice: typing.Optional[CreateChatCompletionStreamRequestToolChoice] = OMIT,
        tools: typing.Optional[typing.Sequence[ToolDefinition]] = OMIT,
        top_logprobs: typing.Optional[int] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[ChatCompletionChunk]:
        """
        Create a Chat Completion.

        Parameters
        ----------
        messages : typing.Sequence[ChatMessage]
            A list of messages comprising the conversation so far.

        model : str
            The identifier of the model to use.Can be a shared tenancy or custom model identifier.

        frequency_penalty : typing.Optional[float]
            Penalizes new tokens based on their frequency in the generated text so far.

        ignore_eos : typing.Optional[bool]
            Whether to ignore the EOS token and continue generating tokens after the EOS token is generated.

        logit_bias : typing.Optional[typing.Dict[str, typing.Optional[float]]]
            Modify the likelihood of specified tokens appearing in the completion. Accepts a JSON object that maps tokens (specified by their token ID in the tokenizer) to an associated bias value from -100 to 100. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model, but values between -1 and 1 should decrease or increase likelihood of selection; values like -100 or 100 should result in a ban or exclusive selection of the relevant token. As an example, you can pass {'50256': -100} to prevent the <|endoftext|> token from being generated.

        loglikelihood : typing.Optional[bool]
            Return log probabilities for all prompt tokens excluding the first one from prefill step if True.

        logprobs : typing.Optional[bool]
            Whether to return log probabilities of the output tokens or not. If true, returns the log probabilities of each output token returned in the content of message.

        max_tokens : typing.Optional[int]
            Maximum number of tokens to generate per output sequence.

        n : typing.Optional[int]
            Number of output sequences to return.

        peft : typing.Optional[str]
            Parameter-efficient fine-tuning ID.

        presence_penalty : typing.Optional[float]
            Penalizes new tokens based on whether they appear in the generated text so far.

        repetition_penalty : typing.Optional[float]
            Controls the likelihood of the model generating repeated texts.

        response_format : typing.Optional[ChatCompletionResponseFormat]
            Allows specification of a response format and associated schema that will constrain the LLM output to that structure. For example, using the `json_object` type allows you to provide a desired json schema for the output to follow.

        stop : typing.Optional[Stop]
            Generation stop condition.

        stream_options : typing.Optional[StreamOptions]
            If set, usageStats will be streamed on the last content-containing chunk.

        temperature : typing.Optional[float]
            Controls the randomness of the sampling.

        tool_choice : typing.Optional[CreateChatCompletionStreamRequestToolChoice]
            Controls which (if any) tool is called by the model. none means the model will not call any tool and instead generates a message. auto means the model can pick between generating a message or calling one or more tools. required means the model must call one or more tools. Specifying a particular tool via {"type": "function", "function": {"name": "my_function"}} forces the model to call that tool. none is the default when no tools are present. auto is the default if tools are present.

        tools : typing.Optional[typing.Sequence[ToolDefinition]]
            A list of tools the model may call. Currently, only functions are supported as a tool. Use this to provide a list of functions the model may generate JSON inputs for. A max of 128 functions are supported.

        top_logprobs : typing.Optional[int]
            An integer between 0 and 20 specifying the number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to `true` if this parameter is used.

        top_p : typing.Optional[float]
            Controls the cumulative probability of the top tokens to consider.

        user : typing.Optional[str]
            A unique identifier representing your end-user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[ChatCompletionChunk]


        Examples
        --------
        from octoai.client import OctoAI
        from octoai.text_gen import (
            ChatCompletionResponseFormat,
            ChatMessage,
            FunctionDefinition,
            StreamOptions,
            ToolCall,
            ToolDefinition,
        )

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        response = client.text_gen.create_chat_completion_stream(
            frequency_penalty=1.1,
            ignore_eos=True,
            logit_bias={"string": {"key": "value"}},
            loglikelihood=True,
            logprobs=True,
            max_tokens=1,
            messages=[
                ChatMessage(
                    content="string",
                    role="string",
                    tool_calls=[ToolCall()],
                )
            ],
            model="string",
            n=1,
            peft="string",
            presence_penalty=1.1,
            repetition_penalty=1.1,
            response_format=ChatCompletionResponseFormat(
                schema={"string": {"key": "value"}},
                type="string",
            ),
            stop="string",
            stream_options=StreamOptions(
                include_usage=True,
            ),
            temperature=1.1,
            tool_choice={"key": "value"},
            tools=[
                ToolDefinition(
                    function=FunctionDefinition(
                        description="string",
                        name="string",
                        parameters={"string": {"key": "value"}},
                    ),
                    type={"key": "value"},
                )
            ],
            top_logprobs=1,
            top_p=1.1,
            user="string",
        )
        for chunk in response:
            yield chunk
        """
        with self._client_wrapper.httpx_client.stream(
            "v1/chat/completions",
            base_url=self._client_wrapper.get_environment().text_gen,
            method="POST",
            json={
                "frequency_penalty": frequency_penalty,
                "ignore_eos": ignore_eos,
                "logit_bias": logit_bias,
                "loglikelihood": loglikelihood,
                "logprobs": logprobs,
                "max_tokens": max_tokens,
                "messages": messages,
                "model": model,
                "n": n,
                "peft": peft,
                "presence_penalty": presence_penalty,
                "repetition_penalty": repetition_penalty,
                "response_format": response_format,
                "stop": stop,
                "stream_options": stream_options,
                "temperature": temperature,
                "tool_choice": tool_choice,
                "tools": tools,
                "top_logprobs": top_logprobs,
                "top_p": top_p,
                "user": user,
                "stream": True,
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:
            try:
                if 200 <= _response.status_code < 300:
                    _event_source = httpx_sse.EventSource(_response)
                    for _sse in _event_source.iter_sse():
                        try:
                            yield pydantic_v1.parse_obj_as(ChatCompletionChunk, json.loads(_sse.data))  # type: ignore
                        except:
                            pass
                    return
                _response.read()
                if _response.status_code == 422:
                    raise UnprocessableEntityError(
                        pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                    )
                if _response.status_code == 500:
                    raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_chat_completion(
        self,
        *,
        messages: typing.Sequence[ChatMessage],
        model: str,
        frequency_penalty: typing.Optional[float] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        loglikelihood: typing.Optional[bool] = OMIT,
        logprobs: typing.Optional[bool] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        n: typing.Optional[int] = OMIT,
        peft: typing.Optional[str] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        response_format: typing.Optional[ChatCompletionResponseFormat] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        stream_options: typing.Optional[StreamOptions] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        tool_choice: typing.Optional[CreateChatCompletionRequestToolChoice] = OMIT,
        tools: typing.Optional[typing.Sequence[ToolDefinition]] = OMIT,
        top_logprobs: typing.Optional[int] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> ChatCompletionResponse:
        """
        Create a Chat Completion.

        Parameters
        ----------
        messages : typing.Sequence[ChatMessage]
            A list of messages comprising the conversation so far.

        model : str
            The identifier of the model to use.Can be a shared tenancy or custom model identifier.

        frequency_penalty : typing.Optional[float]
            Penalizes new tokens based on their frequency in the generated text so far.

        ignore_eos : typing.Optional[bool]
            Whether to ignore the EOS token and continue generating tokens after the EOS token is generated.

        logit_bias : typing.Optional[typing.Dict[str, typing.Optional[float]]]
            Modify the likelihood of specified tokens appearing in the completion. Accepts a JSON object that maps tokens (specified by their token ID in the tokenizer) to an associated bias value from -100 to 100. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model, but values between -1 and 1 should decrease or increase likelihood of selection; values like -100 or 100 should result in a ban or exclusive selection of the relevant token. As an example, you can pass {'50256': -100} to prevent the <|endoftext|> token from being generated.

        loglikelihood : typing.Optional[bool]
            Return log probabilities for all prompt tokens excluding the first one from prefill step if True.

        logprobs : typing.Optional[bool]
            Whether to return log probabilities of the output tokens or not. If true, returns the log probabilities of each output token returned in the content of message.

        max_tokens : typing.Optional[int]
            Maximum number of tokens to generate per output sequence.

        n : typing.Optional[int]
            Number of output sequences to return.

        peft : typing.Optional[str]
            Parameter-efficient fine-tuning ID.

        presence_penalty : typing.Optional[float]
            Penalizes new tokens based on whether they appear in the generated text so far.

        repetition_penalty : typing.Optional[float]
            Controls the likelihood of the model generating repeated texts.

        response_format : typing.Optional[ChatCompletionResponseFormat]
            Allows specification of a response format and associated schema that will constrain the LLM output to that structure. For example, using the `json_object` type allows you to provide a desired json schema for the output to follow.

        stop : typing.Optional[Stop]
            Generation stop condition.

        stream_options : typing.Optional[StreamOptions]
            If set, usageStats will be streamed on the last content-containing chunk.

        temperature : typing.Optional[float]
            Controls the randomness of the sampling.

        tool_choice : typing.Optional[CreateChatCompletionRequestToolChoice]
            Controls which (if any) tool is called by the model. none means the model will not call any tool and instead generates a message. auto means the model can pick between generating a message or calling one or more tools. required means the model must call one or more tools. Specifying a particular tool via {"type": "function", "function": {"name": "my_function"}} forces the model to call that tool. none is the default when no tools are present. auto is the default if tools are present.

        tools : typing.Optional[typing.Sequence[ToolDefinition]]
            A list of tools the model may call. Currently, only functions are supported as a tool. Use this to provide a list of functions the model may generate JSON inputs for. A max of 128 functions are supported.

        top_logprobs : typing.Optional[int]
            An integer between 0 and 20 specifying the number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to `true` if this parameter is used.

        top_p : typing.Optional[float]
            Controls the cumulative probability of the top tokens to consider.

        user : typing.Optional[str]
            A unique identifier representing your end-user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChatCompletionResponse


        Examples
        --------
        from octoai.client import OctoAI
        from octoai.text_gen import ChatMessage

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.text_gen.create_chat_completion(
            messages=[
                ChatMessage(
                    role="role",
                )
            ],
            model="model",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/chat/completions",
            base_url=self._client_wrapper.get_environment().text_gen,
            method="POST",
            json={
                "frequency_penalty": frequency_penalty,
                "ignore_eos": ignore_eos,
                "logit_bias": logit_bias,
                "loglikelihood": loglikelihood,
                "logprobs": logprobs,
                "max_tokens": max_tokens,
                "messages": messages,
                "model": model,
                "n": n,
                "peft": peft,
                "presence_penalty": presence_penalty,
                "repetition_penalty": repetition_penalty,
                "response_format": response_format,
                "stop": stop,
                "stream_options": stream_options,
                "temperature": temperature,
                "tool_choice": tool_choice,
                "tools": tools,
                "top_logprobs": top_logprobs,
                "top_p": top_p,
                "user": user,
                "stream": False,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ChatCompletionResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            if _response.status_code == 500:
                raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_completion_stream(
        self,
        *,
        model: str,
        best_of: typing.Optional[int] = OMIT,
        echo: typing.Optional[bool] = OMIT,
        frequency_penalty: typing.Optional[float] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        loglikelihood: typing.Optional[bool] = OMIT,
        logprobs: typing.Optional[int] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        n: typing.Optional[int] = OMIT,
        peft: typing.Optional[str] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        prompt: typing.Optional[Prompt] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        seed: typing.Optional[int] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        stream_options: typing.Optional[StreamOptions] = OMIT,
        suffix: typing.Optional[str] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[CompletionResponse]:
        """
        Parameters
        ----------
        model : str
            Model name to use for completion.

        best_of : typing.Optional[int]
            Number of sequences that are generated from the prompt.`best_of` must be greater than or equal to `n`.

        echo : typing.Optional[bool]
            Echo back the prompt in addition to the completion.

        frequency_penalty : typing.Optional[float]
            Penalizes new tokens based on their frequency in the generated text so far.

        ignore_eos : typing.Optional[bool]
            Whether to ignore the EOS token and continue generating tokens after the EOS token is generated.

        logit_bias : typing.Optional[typing.Dict[str, typing.Optional[float]]]
            Modify the likelihood of specified tokens appearing in the completion.

        loglikelihood : typing.Optional[bool]
            Switch on loglikelihood regime and return log probabilities from all prompt tokens from prefill.

        logprobs : typing.Optional[int]
            Number of log probabilities to return per output token.

        max_tokens : typing.Optional[int]
            Maximum number of tokens to generate per output sequence.

        n : typing.Optional[int]
            Number of output sequences to return.

        peft : typing.Optional[str]
            Parameter-efficient fine-tuning ID.

        presence_penalty : typing.Optional[float]
            Penalizes new tokens based on whether they appear in the generated text so far.

        prompt : typing.Optional[Prompt]
            The prompt to generate completions from.

        repetition_penalty : typing.Optional[float]
            Controls the likelihood of the model generating repeated texts.

        seed : typing.Optional[int]
            If specified, our system will make a best effort to sample deterministically, such that repeated requests with the same `seed` and parameters should return the same result. Determinism is not guaranteed, and you should refer to the `system_fingerprint` response parameter to monitor changes in the backend.

        stop : typing.Optional[Stop]
            Strings that stop the generation when they are generated.

        stream_options : typing.Optional[StreamOptions]
            If set, usageStats will be streamed on the last content-containing chunk.

        suffix : typing.Optional[str]
            The suffix that comes after a completion of inserted text.

        temperature : typing.Optional[float]
            Controls the randomness of the sampling.

        top_p : typing.Optional[float]
            Controls the cumulative probability of the top tokens to consider.

        user : typing.Optional[str]
            A unique identifier representing your end-user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[CompletionResponse]


        Examples
        --------
        from octoai.client import OctoAI
        from octoai.text_gen import StreamOptions

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        response = client.text_gen.create_completion_stream(
            best_of=1,
            echo=True,
            frequency_penalty=1.1,
            ignore_eos=True,
            logit_bias={"string": {"key": "value"}},
            loglikelihood=True,
            logprobs=1,
            max_tokens=1,
            model="string",
            n=1,
            peft="string",
            presence_penalty=1.1,
            prompt="string",
            repetition_penalty=1.1,
            seed=1,
            stop="string",
            stream_options=StreamOptions(
                include_usage=True,
            ),
            suffix="string",
            temperature=1.1,
            top_p=1.1,
            user="string",
        )
        for chunk in response:
            yield chunk
        """
        with self._client_wrapper.httpx_client.stream(
            "v1/completions",
            base_url=self._client_wrapper.get_environment().text_gen,
            method="POST",
            json={
                "best_of": best_of,
                "echo": echo,
                "frequency_penalty": frequency_penalty,
                "ignore_eos": ignore_eos,
                "logit_bias": logit_bias,
                "loglikelihood": loglikelihood,
                "logprobs": logprobs,
                "max_tokens": max_tokens,
                "model": model,
                "n": n,
                "peft": peft,
                "presence_penalty": presence_penalty,
                "prompt": prompt,
                "repetition_penalty": repetition_penalty,
                "seed": seed,
                "stop": stop,
                "stream_options": stream_options,
                "suffix": suffix,
                "temperature": temperature,
                "top_p": top_p,
                "user": user,
                "stream": True,
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:
            try:
                if 200 <= _response.status_code < 300:
                    _event_source = httpx_sse.EventSource(_response)
                    for _sse in _event_source.iter_sse():
                        try:
                            yield pydantic_v1.parse_obj_as(CompletionResponse, json.loads(_sse.data))  # type: ignore
                        except:
                            pass
                    return
                _response.read()
                if _response.status_code == 422:
                    raise UnprocessableEntityError(
                        pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                    )
                if _response.status_code == 500:
                    raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_completion(
        self,
        *,
        model: str,
        best_of: typing.Optional[int] = OMIT,
        echo: typing.Optional[bool] = OMIT,
        frequency_penalty: typing.Optional[float] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        loglikelihood: typing.Optional[bool] = OMIT,
        logprobs: typing.Optional[int] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        n: typing.Optional[int] = OMIT,
        peft: typing.Optional[str] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        prompt: typing.Optional[Prompt] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        seed: typing.Optional[int] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        stream_options: typing.Optional[StreamOptions] = OMIT,
        suffix: typing.Optional[str] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> CompletionResponse:
        """
        Parameters
        ----------
        model : str
            Model name to use for completion.

        best_of : typing.Optional[int]
            Number of sequences that are generated from the prompt.`best_of` must be greater than or equal to `n`.

        echo : typing.Optional[bool]
            Echo back the prompt in addition to the completion.

        frequency_penalty : typing.Optional[float]
            Penalizes new tokens based on their frequency in the generated text so far.

        ignore_eos : typing.Optional[bool]
            Whether to ignore the EOS token and continue generating tokens after the EOS token is generated.

        logit_bias : typing.Optional[typing.Dict[str, typing.Optional[float]]]
            Modify the likelihood of specified tokens appearing in the completion.

        loglikelihood : typing.Optional[bool]
            Switch on loglikelihood regime and return log probabilities from all prompt tokens from prefill.

        logprobs : typing.Optional[int]
            Number of log probabilities to return per output token.

        max_tokens : typing.Optional[int]
            Maximum number of tokens to generate per output sequence.

        n : typing.Optional[int]
            Number of output sequences to return.

        peft : typing.Optional[str]
            Parameter-efficient fine-tuning ID.

        presence_penalty : typing.Optional[float]
            Penalizes new tokens based on whether they appear in the generated text so far.

        prompt : typing.Optional[Prompt]
            The prompt to generate completions from.

        repetition_penalty : typing.Optional[float]
            Controls the likelihood of the model generating repeated texts.

        seed : typing.Optional[int]
            If specified, our system will make a best effort to sample deterministically, such that repeated requests with the same `seed` and parameters should return the same result. Determinism is not guaranteed, and you should refer to the `system_fingerprint` response parameter to monitor changes in the backend.

        stop : typing.Optional[Stop]
            Strings that stop the generation when they are generated.

        stream_options : typing.Optional[StreamOptions]
            If set, usageStats will be streamed on the last content-containing chunk.

        suffix : typing.Optional[str]
            The suffix that comes after a completion of inserted text.

        temperature : typing.Optional[float]
            Controls the randomness of the sampling.

        top_p : typing.Optional[float]
            Controls the cumulative probability of the top tokens to consider.

        user : typing.Optional[str]
            A unique identifier representing your end-user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CompletionResponse


        Examples
        --------
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.text_gen.create_completion(
            model="model",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/completions",
            base_url=self._client_wrapper.get_environment().text_gen,
            method="POST",
            json={
                "best_of": best_of,
                "echo": echo,
                "frequency_penalty": frequency_penalty,
                "ignore_eos": ignore_eos,
                "logit_bias": logit_bias,
                "loglikelihood": loglikelihood,
                "logprobs": logprobs,
                "max_tokens": max_tokens,
                "model": model,
                "n": n,
                "peft": peft,
                "presence_penalty": presence_penalty,
                "prompt": prompt,
                "repetition_penalty": repetition_penalty,
                "seed": seed,
                "stop": stop,
                "stream_options": stream_options,
                "suffix": suffix,
                "temperature": temperature,
                "top_p": top_p,
                "user": user,
                "stream": False,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(CompletionResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            if _response.status_code == 500:
                raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTextGenClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_chat_completion_stream(
        self,
        *,
        messages: typing.Sequence[ChatMessage],
        model: str,
        frequency_penalty: typing.Optional[float] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        loglikelihood: typing.Optional[bool] = OMIT,
        logprobs: typing.Optional[bool] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        n: typing.Optional[int] = OMIT,
        peft: typing.Optional[str] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        response_format: typing.Optional[ChatCompletionResponseFormat] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        stream_options: typing.Optional[StreamOptions] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        tool_choice: typing.Optional[CreateChatCompletionStreamRequestToolChoice] = OMIT,
        tools: typing.Optional[typing.Sequence[ToolDefinition]] = OMIT,
        top_logprobs: typing.Optional[int] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[ChatCompletionChunk]:
        """
        Create a Chat Completion.

        Parameters
        ----------
        messages : typing.Sequence[ChatMessage]
            A list of messages comprising the conversation so far.

        model : str
            The identifier of the model to use.Can be a shared tenancy or custom model identifier.

        frequency_penalty : typing.Optional[float]
            Penalizes new tokens based on their frequency in the generated text so far.

        ignore_eos : typing.Optional[bool]
            Whether to ignore the EOS token and continue generating tokens after the EOS token is generated.

        logit_bias : typing.Optional[typing.Dict[str, typing.Optional[float]]]
            Modify the likelihood of specified tokens appearing in the completion. Accepts a JSON object that maps tokens (specified by their token ID in the tokenizer) to an associated bias value from -100 to 100. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model, but values between -1 and 1 should decrease or increase likelihood of selection; values like -100 or 100 should result in a ban or exclusive selection of the relevant token. As an example, you can pass {'50256': -100} to prevent the <|endoftext|> token from being generated.

        loglikelihood : typing.Optional[bool]
            Return log probabilities for all prompt tokens excluding the first one from prefill step if True.

        logprobs : typing.Optional[bool]
            Whether to return log probabilities of the output tokens or not. If true, returns the log probabilities of each output token returned in the content of message.

        max_tokens : typing.Optional[int]
            Maximum number of tokens to generate per output sequence.

        n : typing.Optional[int]
            Number of output sequences to return.

        peft : typing.Optional[str]
            Parameter-efficient fine-tuning ID.

        presence_penalty : typing.Optional[float]
            Penalizes new tokens based on whether they appear in the generated text so far.

        repetition_penalty : typing.Optional[float]
            Controls the likelihood of the model generating repeated texts.

        response_format : typing.Optional[ChatCompletionResponseFormat]
            Allows specification of a response format and associated schema that will constrain the LLM output to that structure. For example, using the `json_object` type allows you to provide a desired json schema for the output to follow.

        stop : typing.Optional[Stop]
            Generation stop condition.

        stream_options : typing.Optional[StreamOptions]
            If set, usageStats will be streamed on the last content-containing chunk.

        temperature : typing.Optional[float]
            Controls the randomness of the sampling.

        tool_choice : typing.Optional[CreateChatCompletionStreamRequestToolChoice]
            Controls which (if any) tool is called by the model. none means the model will not call any tool and instead generates a message. auto means the model can pick between generating a message or calling one or more tools. required means the model must call one or more tools. Specifying a particular tool via {"type": "function", "function": {"name": "my_function"}} forces the model to call that tool. none is the default when no tools are present. auto is the default if tools are present.

        tools : typing.Optional[typing.Sequence[ToolDefinition]]
            A list of tools the model may call. Currently, only functions are supported as a tool. Use this to provide a list of functions the model may generate JSON inputs for. A max of 128 functions are supported.

        top_logprobs : typing.Optional[int]
            An integer between 0 and 20 specifying the number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to `true` if this parameter is used.

        top_p : typing.Optional[float]
            Controls the cumulative probability of the top tokens to consider.

        user : typing.Optional[str]
            A unique identifier representing your end-user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[ChatCompletionChunk]


        Examples
        --------
        from octoai.client import AsyncOctoAI
        from octoai.text_gen import (
            ChatCompletionResponseFormat,
            ChatMessage,
            FunctionDefinition,
            StreamOptions,
            ToolCall,
            ToolDefinition,
        )

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        response = await client.text_gen.create_chat_completion_stream(
            frequency_penalty=1.1,
            ignore_eos=True,
            logit_bias={"string": {"key": "value"}},
            loglikelihood=True,
            logprobs=True,
            max_tokens=1,
            messages=[
                ChatMessage(
                    content="string",
                    role="string",
                    tool_calls=[ToolCall()],
                )
            ],
            model="string",
            n=1,
            peft="string",
            presence_penalty=1.1,
            repetition_penalty=1.1,
            response_format=ChatCompletionResponseFormat(
                schema={"string": {"key": "value"}},
                type="string",
            ),
            stop="string",
            stream_options=StreamOptions(
                include_usage=True,
            ),
            temperature=1.1,
            tool_choice={"key": "value"},
            tools=[
                ToolDefinition(
                    function=FunctionDefinition(
                        description="string",
                        name="string",
                        parameters={"string": {"key": "value"}},
                    ),
                    type={"key": "value"},
                )
            ],
            top_logprobs=1,
            top_p=1.1,
            user="string",
        )
        async for chunk in response:
            yield chunk
        """
        async with self._client_wrapper.httpx_client.stream(
            "v1/chat/completions",
            base_url=self._client_wrapper.get_environment().text_gen,
            method="POST",
            json={
                "frequency_penalty": frequency_penalty,
                "ignore_eos": ignore_eos,
                "logit_bias": logit_bias,
                "loglikelihood": loglikelihood,
                "logprobs": logprobs,
                "max_tokens": max_tokens,
                "messages": messages,
                "model": model,
                "n": n,
                "peft": peft,
                "presence_penalty": presence_penalty,
                "repetition_penalty": repetition_penalty,
                "response_format": response_format,
                "stop": stop,
                "stream_options": stream_options,
                "temperature": temperature,
                "tool_choice": tool_choice,
                "tools": tools,
                "top_logprobs": top_logprobs,
                "top_p": top_p,
                "user": user,
                "stream": True,
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:
            try:
                if 200 <= _response.status_code < 300:
                    _event_source = httpx_sse.EventSource(_response)
                    async for _sse in _event_source.aiter_sse():
                        try:
                            yield pydantic_v1.parse_obj_as(ChatCompletionChunk, json.loads(_sse.data))  # type: ignore
                        except:
                            pass
                    return
                await _response.aread()
                if _response.status_code == 422:
                    raise UnprocessableEntityError(
                        pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                    )
                if _response.status_code == 500:
                    raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_chat_completion(
        self,
        *,
        messages: typing.Sequence[ChatMessage],
        model: str,
        frequency_penalty: typing.Optional[float] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        loglikelihood: typing.Optional[bool] = OMIT,
        logprobs: typing.Optional[bool] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        n: typing.Optional[int] = OMIT,
        peft: typing.Optional[str] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        response_format: typing.Optional[ChatCompletionResponseFormat] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        stream_options: typing.Optional[StreamOptions] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        tool_choice: typing.Optional[CreateChatCompletionRequestToolChoice] = OMIT,
        tools: typing.Optional[typing.Sequence[ToolDefinition]] = OMIT,
        top_logprobs: typing.Optional[int] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> ChatCompletionResponse:
        """
        Create a Chat Completion.

        Parameters
        ----------
        messages : typing.Sequence[ChatMessage]
            A list of messages comprising the conversation so far.

        model : str
            The identifier of the model to use.Can be a shared tenancy or custom model identifier.

        frequency_penalty : typing.Optional[float]
            Penalizes new tokens based on their frequency in the generated text so far.

        ignore_eos : typing.Optional[bool]
            Whether to ignore the EOS token and continue generating tokens after the EOS token is generated.

        logit_bias : typing.Optional[typing.Dict[str, typing.Optional[float]]]
            Modify the likelihood of specified tokens appearing in the completion. Accepts a JSON object that maps tokens (specified by their token ID in the tokenizer) to an associated bias value from -100 to 100. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model, but values between -1 and 1 should decrease or increase likelihood of selection; values like -100 or 100 should result in a ban or exclusive selection of the relevant token. As an example, you can pass {'50256': -100} to prevent the <|endoftext|> token from being generated.

        loglikelihood : typing.Optional[bool]
            Return log probabilities for all prompt tokens excluding the first one from prefill step if True.

        logprobs : typing.Optional[bool]
            Whether to return log probabilities of the output tokens or not. If true, returns the log probabilities of each output token returned in the content of message.

        max_tokens : typing.Optional[int]
            Maximum number of tokens to generate per output sequence.

        n : typing.Optional[int]
            Number of output sequences to return.

        peft : typing.Optional[str]
            Parameter-efficient fine-tuning ID.

        presence_penalty : typing.Optional[float]
            Penalizes new tokens based on whether they appear in the generated text so far.

        repetition_penalty : typing.Optional[float]
            Controls the likelihood of the model generating repeated texts.

        response_format : typing.Optional[ChatCompletionResponseFormat]
            Allows specification of a response format and associated schema that will constrain the LLM output to that structure. For example, using the `json_object` type allows you to provide a desired json schema for the output to follow.

        stop : typing.Optional[Stop]
            Generation stop condition.

        stream_options : typing.Optional[StreamOptions]
            If set, usageStats will be streamed on the last content-containing chunk.

        temperature : typing.Optional[float]
            Controls the randomness of the sampling.

        tool_choice : typing.Optional[CreateChatCompletionRequestToolChoice]
            Controls which (if any) tool is called by the model. none means the model will not call any tool and instead generates a message. auto means the model can pick between generating a message or calling one or more tools. required means the model must call one or more tools. Specifying a particular tool via {"type": "function", "function": {"name": "my_function"}} forces the model to call that tool. none is the default when no tools are present. auto is the default if tools are present.

        tools : typing.Optional[typing.Sequence[ToolDefinition]]
            A list of tools the model may call. Currently, only functions are supported as a tool. Use this to provide a list of functions the model may generate JSON inputs for. A max of 128 functions are supported.

        top_logprobs : typing.Optional[int]
            An integer between 0 and 20 specifying the number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to `true` if this parameter is used.

        top_p : typing.Optional[float]
            Controls the cumulative probability of the top tokens to consider.

        user : typing.Optional[str]
            A unique identifier representing your end-user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChatCompletionResponse


        Examples
        --------
        from octoai.client import AsyncOctoAI
        from octoai.text_gen import ChatMessage

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.text_gen.create_chat_completion(
            messages=[
                ChatMessage(
                    role="role",
                )
            ],
            model="model",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/chat/completions",
            base_url=self._client_wrapper.get_environment().text_gen,
            method="POST",
            json={
                "frequency_penalty": frequency_penalty,
                "ignore_eos": ignore_eos,
                "logit_bias": logit_bias,
                "loglikelihood": loglikelihood,
                "logprobs": logprobs,
                "max_tokens": max_tokens,
                "messages": messages,
                "model": model,
                "n": n,
                "peft": peft,
                "presence_penalty": presence_penalty,
                "repetition_penalty": repetition_penalty,
                "response_format": response_format,
                "stop": stop,
                "stream_options": stream_options,
                "temperature": temperature,
                "tool_choice": tool_choice,
                "tools": tools,
                "top_logprobs": top_logprobs,
                "top_p": top_p,
                "user": user,
                "stream": False,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ChatCompletionResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            if _response.status_code == 500:
                raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_completion_stream(
        self,
        *,
        model: str,
        best_of: typing.Optional[int] = OMIT,
        echo: typing.Optional[bool] = OMIT,
        frequency_penalty: typing.Optional[float] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        loglikelihood: typing.Optional[bool] = OMIT,
        logprobs: typing.Optional[int] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        n: typing.Optional[int] = OMIT,
        peft: typing.Optional[str] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        prompt: typing.Optional[Prompt] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        seed: typing.Optional[int] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        stream_options: typing.Optional[StreamOptions] = OMIT,
        suffix: typing.Optional[str] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[CompletionResponse]:
        """
        Parameters
        ----------
        model : str
            Model name to use for completion.

        best_of : typing.Optional[int]
            Number of sequences that are generated from the prompt.`best_of` must be greater than or equal to `n`.

        echo : typing.Optional[bool]
            Echo back the prompt in addition to the completion.

        frequency_penalty : typing.Optional[float]
            Penalizes new tokens based on their frequency in the generated text so far.

        ignore_eos : typing.Optional[bool]
            Whether to ignore the EOS token and continue generating tokens after the EOS token is generated.

        logit_bias : typing.Optional[typing.Dict[str, typing.Optional[float]]]
            Modify the likelihood of specified tokens appearing in the completion.

        loglikelihood : typing.Optional[bool]
            Switch on loglikelihood regime and return log probabilities from all prompt tokens from prefill.

        logprobs : typing.Optional[int]
            Number of log probabilities to return per output token.

        max_tokens : typing.Optional[int]
            Maximum number of tokens to generate per output sequence.

        n : typing.Optional[int]
            Number of output sequences to return.

        peft : typing.Optional[str]
            Parameter-efficient fine-tuning ID.

        presence_penalty : typing.Optional[float]
            Penalizes new tokens based on whether they appear in the generated text so far.

        prompt : typing.Optional[Prompt]
            The prompt to generate completions from.

        repetition_penalty : typing.Optional[float]
            Controls the likelihood of the model generating repeated texts.

        seed : typing.Optional[int]
            If specified, our system will make a best effort to sample deterministically, such that repeated requests with the same `seed` and parameters should return the same result. Determinism is not guaranteed, and you should refer to the `system_fingerprint` response parameter to monitor changes in the backend.

        stop : typing.Optional[Stop]
            Strings that stop the generation when they are generated.

        stream_options : typing.Optional[StreamOptions]
            If set, usageStats will be streamed on the last content-containing chunk.

        suffix : typing.Optional[str]
            The suffix that comes after a completion of inserted text.

        temperature : typing.Optional[float]
            Controls the randomness of the sampling.

        top_p : typing.Optional[float]
            Controls the cumulative probability of the top tokens to consider.

        user : typing.Optional[str]
            A unique identifier representing your end-user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[CompletionResponse]


        Examples
        --------
        from octoai.client import AsyncOctoAI
        from octoai.text_gen import StreamOptions

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        response = await client.text_gen.create_completion_stream(
            best_of=1,
            echo=True,
            frequency_penalty=1.1,
            ignore_eos=True,
            logit_bias={"string": {"key": "value"}},
            loglikelihood=True,
            logprobs=1,
            max_tokens=1,
            model="string",
            n=1,
            peft="string",
            presence_penalty=1.1,
            prompt="string",
            repetition_penalty=1.1,
            seed=1,
            stop="string",
            stream_options=StreamOptions(
                include_usage=True,
            ),
            suffix="string",
            temperature=1.1,
            top_p=1.1,
            user="string",
        )
        async for chunk in response:
            yield chunk
        """
        async with self._client_wrapper.httpx_client.stream(
            "v1/completions",
            base_url=self._client_wrapper.get_environment().text_gen,
            method="POST",
            json={
                "best_of": best_of,
                "echo": echo,
                "frequency_penalty": frequency_penalty,
                "ignore_eos": ignore_eos,
                "logit_bias": logit_bias,
                "loglikelihood": loglikelihood,
                "logprobs": logprobs,
                "max_tokens": max_tokens,
                "model": model,
                "n": n,
                "peft": peft,
                "presence_penalty": presence_penalty,
                "prompt": prompt,
                "repetition_penalty": repetition_penalty,
                "seed": seed,
                "stop": stop,
                "stream_options": stream_options,
                "suffix": suffix,
                "temperature": temperature,
                "top_p": top_p,
                "user": user,
                "stream": True,
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:
            try:
                if 200 <= _response.status_code < 300:
                    _event_source = httpx_sse.EventSource(_response)
                    async for _sse in _event_source.aiter_sse():
                        try:
                            yield pydantic_v1.parse_obj_as(CompletionResponse, json.loads(_sse.data))  # type: ignore
                        except:
                            pass
                    return
                await _response.aread()
                if _response.status_code == 422:
                    raise UnprocessableEntityError(
                        pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                    )
                if _response.status_code == 500:
                    raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_completion(
        self,
        *,
        model: str,
        best_of: typing.Optional[int] = OMIT,
        echo: typing.Optional[bool] = OMIT,
        frequency_penalty: typing.Optional[float] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        loglikelihood: typing.Optional[bool] = OMIT,
        logprobs: typing.Optional[int] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        n: typing.Optional[int] = OMIT,
        peft: typing.Optional[str] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        prompt: typing.Optional[Prompt] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        seed: typing.Optional[int] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        stream_options: typing.Optional[StreamOptions] = OMIT,
        suffix: typing.Optional[str] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> CompletionResponse:
        """
        Parameters
        ----------
        model : str
            Model name to use for completion.

        best_of : typing.Optional[int]
            Number of sequences that are generated from the prompt.`best_of` must be greater than or equal to `n`.

        echo : typing.Optional[bool]
            Echo back the prompt in addition to the completion.

        frequency_penalty : typing.Optional[float]
            Penalizes new tokens based on their frequency in the generated text so far.

        ignore_eos : typing.Optional[bool]
            Whether to ignore the EOS token and continue generating tokens after the EOS token is generated.

        logit_bias : typing.Optional[typing.Dict[str, typing.Optional[float]]]
            Modify the likelihood of specified tokens appearing in the completion.

        loglikelihood : typing.Optional[bool]
            Switch on loglikelihood regime and return log probabilities from all prompt tokens from prefill.

        logprobs : typing.Optional[int]
            Number of log probabilities to return per output token.

        max_tokens : typing.Optional[int]
            Maximum number of tokens to generate per output sequence.

        n : typing.Optional[int]
            Number of output sequences to return.

        peft : typing.Optional[str]
            Parameter-efficient fine-tuning ID.

        presence_penalty : typing.Optional[float]
            Penalizes new tokens based on whether they appear in the generated text so far.

        prompt : typing.Optional[Prompt]
            The prompt to generate completions from.

        repetition_penalty : typing.Optional[float]
            Controls the likelihood of the model generating repeated texts.

        seed : typing.Optional[int]
            If specified, our system will make a best effort to sample deterministically, such that repeated requests with the same `seed` and parameters should return the same result. Determinism is not guaranteed, and you should refer to the `system_fingerprint` response parameter to monitor changes in the backend.

        stop : typing.Optional[Stop]
            Strings that stop the generation when they are generated.

        stream_options : typing.Optional[StreamOptions]
            If set, usageStats will be streamed on the last content-containing chunk.

        suffix : typing.Optional[str]
            The suffix that comes after a completion of inserted text.

        temperature : typing.Optional[float]
            Controls the randomness of the sampling.

        top_p : typing.Optional[float]
            Controls the cumulative probability of the top tokens to consider.

        user : typing.Optional[str]
            A unique identifier representing your end-user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CompletionResponse


        Examples
        --------
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.text_gen.create_completion(
            model="model",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/completions",
            base_url=self._client_wrapper.get_environment().text_gen,
            method="POST",
            json={
                "best_of": best_of,
                "echo": echo,
                "frequency_penalty": frequency_penalty,
                "ignore_eos": ignore_eos,
                "logit_bias": logit_bias,
                "loglikelihood": loglikelihood,
                "logprobs": logprobs,
                "max_tokens": max_tokens,
                "model": model,
                "n": n,
                "peft": peft,
                "presence_penalty": presence_penalty,
                "prompt": prompt,
                "repetition_penalty": repetition_penalty,
                "seed": seed,
                "stop": stop,
                "stream_options": stream_options,
                "suffix": suffix,
                "temperature": temperature,
                "top_p": top_p,
                "user": user,
                "stream": False,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(CompletionResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            if _response.status_code == 500:
                raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

AssetType = typing.Union[typing.Literal["file", "checkpoint", "lora", "textual_inversion", "vae", "latent"], typing.Any]



import asyncio
import contextlib
import functools
import pathlib
from time import sleep
import time
import typing

from octoai.asset_library.types.status import Status
from .asset_library.client import AsyncAssetLibraryClient
from .asset_library.types.create_asset_response import CreateAssetResponse
from .asset_library.types.create_asset_response_transfer_api import CreateAssetResponseTransferApi, CreateAssetResponseTransferApi_PresignedUrl, CreateAssetResponseTransferApi_Sts
import boto3
from .core.client_wrapper import AsyncClientWrapper
import httpx
from octoai.asset_library.types.asset import Asset

from .asset_library.client import OMIT
from .asset_library.types.data import Data
from .asset_library.types.file_structure import FileStructure
from .asset_library.types.transfer_api_type import TransferApiType
from .core.request_options import RequestOptions
from .core.client_wrapper import SyncClientWrapper
from .asset_library.client import AssetLibraryClient
from .core.api_error import ApiError


_TERMINAL_STATUSES: typing.Set[Status] = {"ready", "deleted", "rejected", "error"}


class AssetReadyTimeout(Exception):
    pass


class AssetNotReady(Exception):
    pass


class IncompatibleFileStructure(Exception):
    pass


def _sts_upload(x_api: CreateAssetResponseTransferApi_Sts, file_structure: FileStructure, file: typing.Union[str, typing.BinaryIO]):
    is_dir = isinstance(file, str) and pathlib.Path(file).is_dir()

    if is_dir and file_structure != "multiple_files":
        raise IncompatibleFileStructure("Asset's file structure is a multiple_files but file is not a directory")

    s3_client = boto3.client(
        "s3",
        aws_access_key_id=x_api.aws_access_key_id,
        aws_secret_access_key=x_api.aws_secret_access_key,
        aws_session_token=x_api.aws_session_token,
    )

    if isinstance(file, str) and is_dir:
        for file_path in pathlib.Path(file).rglob("*"):
            if file_path.is_file():
                s3_client.upload_file(
                    str(file_path),
                    x_api.s3bucket,
                    f"{x_api.s3key}{file_path.relative_to(file).as_posix()}",
                )
    else:
        try:
            s3_client.upload_file(file, x_api.s3bucket, x_api.s3key)
        except Exception as e:
            raise Exception(f"Error uploading file to server: {e}")


class UploadingAssetLibraryClient(AssetLibraryClient):
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        super().__init__(client_wrapper=client_wrapper)

    def create_from_file(
        self,
        file: typing.Union[str, typing.BinaryIO],
        data: Data,
        name: str,
        description: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        transfer_api_type: typing.Optional[TransferApiType] = OMIT
    ) -> Asset:
        """
        Parameters:
            - file: typing.Union[str, typing.BinaryIO]. File to upload.

            - data: Data. Asset data.

            - name: str. Asset name.

            - description: typing.Optional[str].

            - is_public: typing.Optional[bool]. True if asset is public.

            - transfer_api_type: typing.Optional[TransferApiType]. Transfer API type.
        """
        if isinstance(file, str) and not pathlib.Path(file).exists():
            raise FileNotFoundError(f"File {file} does not exist")

        response: CreateAssetResponse = self.create(
            data=data,
            name=name,
            asset_type=data.asset_type,
            description=description,
            is_public=is_public,
            transfer_api_type=transfer_api_type
        )
        self._transfer_file(response.transfer_api, response.asset.file_structure, file=file)

        self.complete_upload(response.asset.id)

        return self.wait_for_ready(response.asset, transfer_api_type=transfer_api_type)


    def create_from_url(
        self,
        url: str,
        data: Data,
        name: str,
        description: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        transfer_api_type: typing.Optional[TransferApiType] = "sts"
    ) -> Asset:
        """
        Create an asset from a URL.

        Parameters:
            - url: str. URL to create asset from.

            - data: Data. Asset data.

            - name: str. Asset name.

            - description: typing.Optional[str].

            - is_public: typing.Optional[bool]. True if asset is public.

            - transfer_api_type: typing.Optional[TransferApiType]. Transfer API type.
        """
        response: CreateAssetResponse = self.create(
            data=data,
            name=name,
            asset_type=data.asset_type,
            description=description,
            is_public=is_public,
            url=url,
            transfer_api_type=transfer_api_type,
        )

        return self.wait_for_ready(response.asset, transfer_api_type=transfer_api_type)


    def wait_for_ready(
        self,
        asset: Asset,
        poll_interval=10,
        timeout_seconds=900,
        transfer_api_type: typing.Optional[TransferApiType] = OMIT
    ):
        """
        Wait for asset to be ready to use.

        This waits until the asset's status is READY or an error status.

        Parameters:
            - asset: Asset. Asset to wait on.

            - poll_interval: int. Poll interval in seconds.

            - timeout_seconds: int. Timeout in seconds.

            - transfer_api_type: typing.Optional[TransferApiType]. Transfer API type.
        """
        start_time = time.monotonic()

        while asset.status not in _TERMINAL_STATUSES:
            now = time.monotonic()
            if now - start_time > timeout_seconds:
                raise AssetReadyTimeout("Asset creation timed out")

            time.sleep(poll_interval)
            if transfer_api_type is not OMIT:
                asset = self.get(asset.id, transfer_api_type=transfer_api_type).asset
            else:
                asset = self.get(asset.id).asset

        if asset.status != "ready":
            raise AssetNotReady(f"Asset creation failed with status: {asset.status}")

        return asset


    def _transfer_file(self, transfer_api_used: CreateAssetResponseTransferApi, file_structure: FileStructure, file: typing.Union[str, typing.BinaryIO]):
        if isinstance(transfer_api_used, CreateAssetResponseTransferApi_PresignedUrl):
            with contextlib.ExitStack() as exit_stack:
                file_data = exit_stack.enter_context(open(file, "rb")) if isinstance(file, str) else file
                upload_resp = httpx.put(
                    url=transfer_api_used.put_url, content=file_data, timeout=60000
                )
                upload_resp.raise_for_status()
        elif isinstance(transfer_api_used, CreateAssetResponseTransferApi_Sts):
            _sts_upload(transfer_api_used, file_structure, file)


class AsyncUploadingAssetLibraryClient(AsyncAssetLibraryClient):
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        super().__init__(client_wrapper=client_wrapper)
        self._signed_url_client = httpx.AsyncClient()

    async def create_from_file(
        self,
        file: typing.Union[str, typing.BinaryIO],
        data: Data,
        name: str,
        description: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        transfer_api_type: typing.Optional[TransferApiType] = OMIT
    ) -> Asset:
        """
        Parameters:
            - file: typing.Union[str, typing.BinaryIO]. File to upload.

            - data: Data. Asset data.

            - name: str. Asset name.

            - description: typing.Optional[str].

            - is_public: typing.Optional[bool]. True if asset is public.

            - transfer_api_type: typing.Optional[TransferApiType]. Transfer API type.
        """
        if isinstance(file, str) and not pathlib.Path(file).exists():
            raise FileNotFoundError(f"File {file} does not exist")

        response: CreateAssetResponse = await self.create(
            data=data,
            name=name,
            asset_type=data.asset_type,
            description=description,
            is_public=is_public,
            transfer_api_type=transfer_api_type
        )

        await self._transfer_file(response.transfer_api, response.asset.file_structure, file=file)

        await self.complete_upload(response.asset.id)

        return await self.wait_for_ready(response.asset, transfer_api_type=transfer_api_type)

    async def create_from_url(
        self,
        url: str,
        data: Data,
        name: str,
        description: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        transfer_api_type: typing.Optional[TransferApiType] = "sts"
    ) -> Asset:
        """
        Create an asset from a URL.

        Parameters:
            - url: str. URL to create asset from.

            - data: Data. Asset data.

            - name: str. Asset name.

            - description: typing.Optional[str].

            - is_public: typing.Optional[bool]. True if asset is public.

            - transfer_api_type: typing.Optional[TransferApiType]. Transfer API type.
        """
        response: CreateAssetResponse = await self.create(
            data=data,
            name=name,
            asset_type=data.asset_type,
            description=description,
            is_public=is_public,
            url=url,
            transfer_api_type=transfer_api_type,
        )

        return await self.wait_for_ready(response.asset, transfer_api_type=transfer_api_type)

    async def wait_for_ready(
        self,
        asset: Asset,
        poll_interval=10,
        timeout_seconds=900,
        transfer_api_type: typing.Optional[TransferApiType] = OMIT
    ):
        """
        Wait for asset to be ready to use.

        This waits until the asset's status is READY or an error status.

        Parameters:
            - asset: Asset. Asset to wait on.

            - poll_interval: int. Poll interval in seconds.

            - timeout_seconds: int. Timeout in seconds.

            - transfer_api_type: typing.Optional[TransferApiType]. Transfer API type.
        """
        start_time = time.monotonic()

        while asset.status not in _TERMINAL_STATUSES:
            now = time.monotonic()
            if now - start_time > timeout_seconds:
                raise AssetReadyTimeout("Asset creation timed out")

            await asyncio.sleep(poll_interval)
            if transfer_api_type is not OMIT:
                asset = (await self.get(asset.id, transfer_api_type=transfer_api_type)).asset
            else:
                asset = (await self.get(asset.id)).asset

        if asset.status != "ready":
            raise AssetNotReady(f"Asset creation failed with status: {asset.status}")

        return asset

    async def _transfer_file(self, transfer_api_used: CreateAssetResponseTransferApi, file_structure: FileStructure, file: typing.Union[str, typing.BinaryIO]):
        if isinstance(transfer_api_used, CreateAssetResponseTransferApi_PresignedUrl):
            # TODO: Use aiofiles to make this fully async?
            file_data_bytes = await asyncio.get_running_loop().run_in_executor(
                None,
                functools.partial(
                    _read_entire_file,
                    file
                )
            )

            upload_resp = await self._signed_url_client.put(
                url=transfer_api_used.put_url, content=file_data_bytes, timeout=60000
            )
            upload_resp.raise_for_status()
        elif isinstance(transfer_api_used, CreateAssetResponseTransferApi_Sts):
            _sts_upload(transfer_api_used, file_structure, file)


def _read_entire_file(file: typing.Union[str, typing.BinaryIO]) -> bytes:
    if isinstance(file, str):
        with open(file, "rb") as f:
            return f.read()
    else:
        return file.read()


# This file was auto-generated by Fern from our API Definition.

from .base_engine import BaseEngine
from .details import Details, Details_LoraTune, Details_TextToSpeechLatentTune, Details_TextTune
from .http_validation_error import HttpValidationError
from .list_tunes_response import ListTunesResponse
from .lora_tune import LoraTune
from .lora_tune_checkpoint import LoraTuneCheckpoint
from .lora_tune_file import LoraTuneFile
from .text_to_speech_latent_tune import TextToSpeechLatentTune
from .text_tune import TextTune
from .tune import Tune
from .tune_details import TuneDetails, TuneDetails_LoraTune, TuneDetails_TextToSpeechLatentTune, TuneDetails_TextTune
from .tune_result import TuneResult
from .tune_status import TuneStatus
from .tune_type import TuneType
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem

__all__ = [
    "BaseEngine",
    "Details",
    "Details_LoraTune",
    "Details_TextToSpeechLatentTune",
    "Details_TextTune",
    "HttpValidationError",
    "ListTunesResponse",
    "LoraTune",
    "LoraTuneCheckpoint",
    "LoraTuneFile",
    "TextToSpeechLatentTune",
    "TextTune",
    "Tune",
    "TuneDetails",
    "TuneDetails_LoraTune",
    "TuneDetails_TextToSpeechLatentTune",
    "TuneDetails_TextTune",
    "TuneResult",
    "TuneStatus",
    "TuneType",
    "ValidationError",
    "ValidationErrorLocItem",
]



# This file was auto-generated by Fern from our API Definition.

from .internal_server_error import InternalServerError
from .unprocessable_entity_error import UnprocessableEntityError

__all__ = ["InternalServerError", "UnprocessableEntityError"]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    HttpValidationError,
    ImageEncoding,
    ImageGeneration,
    ImageGenerationRequest,
    ImageGenerationRequestSeed,
    ImageGenerationResponse,
    Scheduler,
    SdxlStyles,
    ValidationError,
    ValidationErrorLocItem,
    VideoGeneration,
    VideoGenerationRequestSeed,
    VideoGenerationResponse,
)
from .errors import UnprocessableEntityError

__all__ = [
    "HttpValidationError",
    "ImageEncoding",
    "ImageGeneration",
    "ImageGenerationRequest",
    "ImageGenerationRequestSeed",
    "ImageGenerationResponse",
    "Scheduler",
    "SdxlStyles",
    "UnprocessableEntityError",
    "ValidationError",
    "ValidationErrorLocItem",
    "VideoGeneration",
    "VideoGenerationRequestSeed",
    "VideoGenerationResponse",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations


class OctoAIEnvironment:
    PRODUCTION: OctoAIEnvironment
    SECURE_LINK: OctoAIEnvironment

    def __init__(self, *, default: str, image_gen: str, text_gen: str):
        self.default = default
        self.image_gen = image_gen
        self.text_gen = text_gen


OctoAIEnvironment.PRODUCTION = OctoAIEnvironment(
    default="https://api.octoai.cloud", image_gen="https://image.octoai.run", text_gen="https://text.octoai.run"
)
OctoAIEnvironment.SECURE_LINK = OctoAIEnvironment(
    default="https://api.securelink.octo.ai",
    image_gen="https://image.securelink.octo.ai",
    text_gen="https://text.securelink.octo.ai",
)



# This file was auto-generated by Fern from our API Definition.

from .chat_completion_choice import ChatCompletionChoice
from .chat_completion_chunk import ChatCompletionChunk
from .chat_completion_chunk_choice import ChatCompletionChunkChoice
from .chat_completion_delta import ChatCompletionDelta
from .chat_completion_response import ChatCompletionResponse
from .chat_completion_response_format import ChatCompletionResponseFormat
from .chat_message import ChatMessage
from .completion_choice import CompletionChoice
from .completion_response import CompletionResponse
from .create_chat_completion_request_tool_choice import CreateChatCompletionRequestToolChoice
from .create_chat_completion_stream_request_tool_choice import CreateChatCompletionStreamRequestToolChoice
from .error_response import ErrorResponse
from .error_response_validation_errors_value import ErrorResponseValidationErrorsValue
from .finish_reason import FinishReason
from .function_call import FunctionCall
from .function_choice import FunctionChoice
from .function_definition import FunctionDefinition
from .function_handle import FunctionHandle
from .http_validation_error import HttpValidationError
from .logprobs import Logprobs
from .logprobs_content import LogprobsContent
from .prompt import Prompt
from .stop import Stop
from .stream_options import StreamOptions
from .tool_call import ToolCall
from .tool_definition import ToolDefinition
from .top_logprobs import TopLogprobs
from .usage_stats import UsageStats
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem

__all__ = [
    "ChatCompletionChoice",
    "ChatCompletionChunk",
    "ChatCompletionChunkChoice",
    "ChatCompletionDelta",
    "ChatCompletionResponse",
    "ChatCompletionResponseFormat",
    "ChatMessage",
    "CompletionChoice",
    "CompletionResponse",
    "CreateChatCompletionRequestToolChoice",
    "CreateChatCompletionStreamRequestToolChoice",
    "ErrorResponse",
    "ErrorResponseValidationErrorsValue",
    "FinishReason",
    "FunctionCall",
    "FunctionChoice",
    "FunctionDefinition",
    "FunctionHandle",
    "HttpValidationError",
    "Logprobs",
    "LogprobsContent",
    "Prompt",
    "Stop",
    "StreamOptions",
    "ToolCall",
    "ToolDefinition",
    "TopLogprobs",
    "UsageStats",
    "ValidationError",
    "ValidationErrorLocItem",
]



# This file was auto-generated by Fern from our API Definition.

import typing

FinishReason = typing.Union[
    typing.Literal["stop", "length", "tool_calls", "content_filter", "function_call", "cancelled"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .image_encoding import ImageEncoding
from .image_generation_request_seed import ImageGenerationRequestSeed
from .scheduler import Scheduler
from .sdxl_styles import SdxlStyles


class ImageGenerationRequest(pydantic_v1.BaseModel):
    """
    Generate one or more images based on the given parameters.
    """

    cfg_scale: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    Floating-point number represeting how closely to adhere to prompt description. Must be a positive number no greater than 50.0.
    """

    checkpoint: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] Custom checkpoint to be used during image generation.
    """

    clip_skip: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] Optionally skip later layers of the text encoder. Higher values lead to more abstract interpretations of the prompt.
    """

    controlnet: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] ControlNet to be used during image generation.
    """

    controlnet_conditioning_scale: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] How strong the effect of the controlnet should be.
    """

    controlnet_early_stop: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] If provided, indicates fraction of steps at which to stop applying controlnet. This can be used to sometimes generate better outputs.
    """

    controlnet_image: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] Controlnet image encoded in b64 string for guiding image generation. Required for controlnet engines.
    """

    controlnet_preprocess: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] Whether to apply automatic ControlNet preprocessing.
    """

    height: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Integer representing the height of image to generate. None will default to 512 for SD 1.5 and 1024 for FLUX, SD3, SDXL, and SSD. Supported resolutions (w,h): FLUX=[(512, 512), (576, 1024), (640, 1536), (768, 1024), (768, 1344), (832, 1216), (896, 1152), (1024, 576), (1024, 768), (1024, 1024), (1152, 896), (1216, 832), (1344, 768), (1536, 640)], SD3=[(640, 1536), (768, 1344), (832, 1216), (896, 1152), (1024, 1024), (1152, 896), (1216, 832), (1344, 768), (1536, 640)], SDXL=[(640, 1536), (768, 1344), (832, 1216), (896, 1152), (1024, 1024), (1152, 896), (1216, 832), (1344, 768), (1536, 640)], SD1.5=[(384, 704), (448, 576), (512, 512), (512, 704), (512, 768), (512, 832), (576, 448), (576, 768), (576, 1024), (640, 512), (640, 640), (640, 768), (704, 384), (704, 1216), (768, 512), (768, 576), (768, 1024), (832, 512), (896, 896), (1024, 576), (1024, 768), (1024, 1024), (1024, 1536), (1216, 704), (1536, 1024)], SSD=[(640, 1536), (768, 1344), (832, 1216), (896, 1152), (1024, 1024), (1152, 896), (1216, 832), (1344, 768), (1536, 640)].
    """

    high_noise_frac: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] Floating-point number that defines the fraction of steps to perform with the base model. Used only by SD XL. Must be greater than or equal to 0.0 and less than or equal to 1.0.
    """

    image_encoding: typing.Optional[ImageEncoding] = pydantic_v1.Field(default=None)
    """
    Define which encoding process should be applied before returning the generated image(s).
    """

    init_image: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    [Not supported on FLUX] Starting point image encoded in b64 string for Image to Image generation mode.
    """

    inpaint_full_res: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    When true, upsample 'init_image' and 'mask_image' such that a bounding box drawn around the non-black content of 'mask_image' becomes the full inpainting frame. The bounding box has the same aspect ratio as that specified by ('height', 'width') parameters.
    """

    inpaint_full_res_padding: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    When 'inpaint_full_res' is true, include a frame around the non-black mask area that is this many pixels wide. This serves as a form of context for the inpainting and may improve blending.
    """

    log_prompt: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    OctoAI stores request prompt if True.
    """

    loras: typing.Optional[typing.Dict[str, typing.Optional[float]]] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] A dictionary of LoRAs to apply. LoRAs as keys and their weights (float) as values.
    """

    mask_image: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] b64 encoded mask image for inpainting. White area should indicate where to paint.
    """

    negative_prompt: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    [Not supported on FLUX] Text describing image traits to avoid during generation.
    """

    negative_prompt2: typing.Optional[str] = pydantic_v1.Field(alias="negative_prompt_2", default=None)
    """
    [Not supported on SD3, FLUX] Text with a high level description of things to avoid during generation. Used only by SD XL.
    """

    num_images: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Integer representing how many output images to generate with a single prompt/configuration.
    """

    outpainting: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] Whether the request requires outpainting or not. If so, special preprocessing is applied for better results.
    """

    prompt: str = pydantic_v1.Field()
    """
    Text describing the image content to generate.
    """

    prompt2: typing.Optional[str] = pydantic_v1.Field(alias="prompt_2", default=None)
    """
    [Not supported on SD3, FLUX] Text with a high-level description of the image to generate. Used only by SD XL.
    """

    sampler: typing.Optional[Scheduler] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] Sampler name (also known as 'scheduler') to use during image generation.
    """

    seed: typing.Optional[ImageGenerationRequestSeed] = pydantic_v1.Field(default=None)
    """
    Integer number or list of integers representing the seeds of random generators. Fixing random seed is useful when attempting to generate a specific image. Must be greater than 0 and less than 2^32.
    """

    steps: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Integer repreenting how many steps of diffusion to run. Must be greater than 0 and less than or equal to 200.
    """

    strength: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    [Not supported on FLUX] Floating-point number indicating how much creative the Image to Image generation mode should be. Must be greater than 0 and less than or equal to 1.0.
    """

    style_preset: typing.Optional[SdxlStyles] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] Pre-defined styles used to guide the output image towards a particular style. Pre-defined styles are only supported by SDXL.
    """

    textual_inversions: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] A dictionary of textual inversions to be used during image generation. Textual inversions as keys and trigger words as values.
    """

    transfer_images: typing.Optional[typing.Dict[str, typing.Optional[typing.List[str]]]] = pydantic_v1.Field(
        default=None
    )
    """
    [Not supported on SD3, FLUX] A dictionary containing a mapping of trigger words to a list of sample images which demonstrate the desired object or style to transfer.
    """

    use_refiner: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    [Not supported on SD3, FLUX] Whether to enable and apply the SDXL refiner model to the image generation.
    """

    vae: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    [Not Supported on SD3, FLUX] Custom VAE to be used during image generation.
    """

    width: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Integer representing the width of image to generate. None will default to 512 for SD 1.5 and 1024 for FLUX, SD3, SDXL, and SSD. Supported resolutions (w,h): FLUX=[(512, 512), (576, 1024), (640, 1536), (768, 1024), (768, 1344), (832, 1216), (896, 1152), (1024, 576), (1024, 768), (1024, 1024), (1152, 896), (1216, 832), (1344, 768), (1536, 640)], SD3=[(640, 1536), (768, 1344), (832, 1216), (896, 1152), (1024, 1024), (1152, 896), (1216, 832), (1344, 768), (1536, 640)], SDXL=[(640, 1536), (768, 1344), (832, 1216), (896, 1152), (1024, 1024), (1152, 896), (1216, 832), (1344, 768), (1536, 640)], SD1.5=[(384, 704), (448, 576), (512, 512), (512, 704), (512, 768), (512, 832), (576, 448), (576, 768), (576, 1024), (640, 512), (640, 640), (640, 768), (704, 384), (704, 1216), (768, 512), (768, 576), (768, 1024), (832, 512), (896, 896), (1024, 576), (1024, 768), (1024, 1024), (1024, 1536), (1216, 704), (1536, 1024)], SSD=[(640, 1536), (768, 1344), (832, 1216), (896, 1152), (1024, 1024), (1152, 896), (1216, 832), (1344, 768), (1536, 640)].
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .http_validation_error import HttpValidationError
from .image_encoding import ImageEncoding
from .image_generation import ImageGeneration
from .image_generation_request import ImageGenerationRequest
from .image_generation_request_seed import ImageGenerationRequestSeed
from .image_generation_response import ImageGenerationResponse
from .scheduler import Scheduler
from .sdxl_styles import SdxlStyles
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem
from .video_generation import VideoGeneration
from .video_generation_request_seed import VideoGenerationRequestSeed
from .video_generation_response import VideoGenerationResponse

__all__ = [
    "HttpValidationError",
    "ImageEncoding",
    "ImageGeneration",
    "ImageGenerationRequest",
    "ImageGenerationRequestSeed",
    "ImageGenerationResponse",
    "Scheduler",
    "SdxlStyles",
    "ValidationError",
    "ValidationErrorLocItem",
    "VideoGeneration",
    "VideoGenerationRequestSeed",
    "VideoGenerationResponse",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .error_response_validation_errors_value import ErrorResponseValidationErrorsValue


class ErrorResponse(pydantic_v1.BaseModel):
    """
    An OpenAI API compatible schema for a error response.
    """

    code: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Optional error code.
    """

    message: str = pydantic_v1.Field()
    """
    A human-readable error message.
    """

    object: typing.Optional[typing.Any] = None
    param: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Optional error parameters.
    """

    type: str = pydantic_v1.Field()
    """
    A type identifier for the error.
    """

    validation_errors: typing.Optional[
        typing.Dict[str, typing.Optional[ErrorResponseValidationErrorsValue]]
    ] = pydantic_v1.Field(default=None)
    """
    Validation error in API.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .lora_tune_checkpoint import LoraTuneCheckpoint
from .lora_tune_file import LoraTuneFile


class TuneDetails_LoraTune(pydantic_v1.BaseModel):
    """
    Tune details. This field is used to store a specific tune type configurations.
    """

    base_checkpoint: LoraTuneCheckpoint
    files: typing.List[LoraTuneFile]
    resize_images: typing.Optional[bool] = None
    seed: typing.Optional[int] = None
    steps: int
    trigger_words: typing.List[str]
    tune_type: typing.Literal["lora_tune"] = "lora_tune"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


class TuneDetails_TextToSpeechLatentTune(pydantic_v1.BaseModel):
    """
    Tune details. This field is used to store a specific tune type configurations.
    """

    asset_ids: typing.List[str]
    inline_assets: typing.Optional[bool] = None
    tune_type: typing.Literal["text_to_speech_latent_tune"] = "text_to_speech_latent_tune"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


class TuneDetails_TextTune(pydantic_v1.BaseModel):
    """
    Tune details. This field is used to store a specific tune type configurations.
    """

    checkpoint: str
    epochs: typing.Optional[int] = None
    learning_rate: typing.Optional[float] = None
    lora_rank: typing.Optional[int] = None
    tune_type: typing.Literal["text_tune"] = "text_tune"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


TuneDetails = typing.Union[TuneDetails_LoraTune, TuneDetails_TextToSpeechLatentTune, TuneDetails_TextTune]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from .errors.unprocessable_entity_error import UnprocessableEntityError
from .types.details import Details
from .types.http_validation_error import HttpValidationError
from .types.list_tunes_response import ListTunesResponse
from .types.tune import Tune
from .types.tune_type import TuneType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FineTuningClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        details: Details,
        name: str,
        continue_on_rejection: typing.Optional[bool] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Tune:
        """
        Spawn a tune.

        Parameters
        ----------
        details : Details
            Details of the tune.

        name : str
            The name of the tune.

        continue_on_rejection : typing.Optional[bool]
            Continue fine-tuning if any asset was rejected.

        description : typing.Optional[str]
            Description for the tune.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Tune
            Successful Response

        Examples
        --------
        from octoai.client import OctoAI
        from octoai.fine_tuning import (
            Details_LoraTune,
            LoraTuneCheckpoint,
            LoraTuneFile,
        )

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.fine_tuning.create(
            continue_on_rejection=True,
            description="string",
            details=Details_LoraTune(
                base_checkpoint=LoraTuneCheckpoint(),
                files=[LoraTuneFile()],
                resize_images=True,
                seed=1,
                steps=1,
                trigger_words=["string"],
            ),
            name="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/tune",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={
                "continue_on_rejection": continue_on_rejection,
                "description": description,
                "details": details,
                "name": name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Tune:
        """
        Retrieve a specific tune.

        Parameters
        ----------
        tune_id : str
            The ID of the tune to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Tune
            Successful Response

        Examples
        --------
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.fine_tuning.get(
            tune_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/tune/{jsonable_encoder(tune_id)}",
            base_url=self._client_wrapper.get_environment().default,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Tune:
        """
        Delete the specified tune.

        Parameters
        ----------
        tune_id : str
            The ID of the tune to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Tune
            Successful Response

        Examples
        --------
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.fine_tuning.delete(
            tune_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/tune/{jsonable_encoder(tune_id)}",
            base_url=self._client_wrapper.get_environment().default,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Tune:
        """
        Cancel the specified tune.

        Parameters
        ----------
        tune_id : str
            The ID of the tune to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Tune
            Successful Response

        Examples
        --------
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.fine_tuning.cancel(
            tune_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/tune/{jsonable_encoder(tune_id)}/cancel",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list(
        self,
        *,
        offset: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        tune_type: typing.Optional[TuneType] = None,
        base_checkpoint_id: typing.Optional[str] = None,
        trigger_words: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        engine: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListTunesResponse:
        """
        List all tunes owned by the current user.

        Parameters
        ----------
        offset : typing.Optional[int]
            Offset into the results.

        limit : typing.Optional[int]
            The max number of results to be shown (limit 100).

        name : typing.Optional[str]
            The name of the tune to filter on.

        tune_type : typing.Optional[TuneType]
            The type of the tune to filter on.

        base_checkpoint_id : typing.Optional[str]
            The base checkpoint ID for the LoRA tune.

        trigger_words : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            The trigger words to filter on.

        engine : typing.Optional[str]
            The engine type for listing tunes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListTunesResponse
            Successful Response

        Examples
        --------
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.fine_tuning.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/tunes",
            base_url=self._client_wrapper.get_environment().default,
            method="GET",
            params={
                "offset": offset,
                "limit": limit,
                "name": name,
                "tune_type": tune_type,
                "base_checkpoint_id": base_checkpoint_id,
                "trigger_words": trigger_words,
                "engine": engine,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ListTunesResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncFineTuningClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        details: Details,
        name: str,
        continue_on_rejection: typing.Optional[bool] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Tune:
        """
        Spawn a tune.

        Parameters
        ----------
        details : Details
            Details of the tune.

        name : str
            The name of the tune.

        continue_on_rejection : typing.Optional[bool]
            Continue fine-tuning if any asset was rejected.

        description : typing.Optional[str]
            Description for the tune.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Tune
            Successful Response

        Examples
        --------
        from octoai.client import AsyncOctoAI
        from octoai.fine_tuning import (
            Details_LoraTune,
            LoraTuneCheckpoint,
            LoraTuneFile,
        )

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.fine_tuning.create(
            continue_on_rejection=True,
            description="string",
            details=Details_LoraTune(
                base_checkpoint=LoraTuneCheckpoint(),
                files=[LoraTuneFile()],
                resize_images=True,
                seed=1,
                steps=1,
                trigger_words=["string"],
            ),
            name="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/tune",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={
                "continue_on_rejection": continue_on_rejection,
                "description": description,
                "details": details,
                "name": name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Tune:
        """
        Retrieve a specific tune.

        Parameters
        ----------
        tune_id : str
            The ID of the tune to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Tune
            Successful Response

        Examples
        --------
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.fine_tuning.get(
            tune_id="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/tune/{jsonable_encoder(tune_id)}",
            base_url=self._client_wrapper.get_environment().default,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Tune:
        """
        Delete the specified tune.

        Parameters
        ----------
        tune_id : str
            The ID of the tune to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Tune
            Successful Response

        Examples
        --------
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.fine_tuning.delete(
            tune_id="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/tune/{jsonable_encoder(tune_id)}",
            base_url=self._client_wrapper.get_environment().default,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Tune:
        """
        Cancel the specified tune.

        Parameters
        ----------
        tune_id : str
            The ID of the tune to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Tune
            Successful Response

        Examples
        --------
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.fine_tuning.cancel(
            tune_id="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/tune/{jsonable_encoder(tune_id)}/cancel",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list(
        self,
        *,
        offset: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        tune_type: typing.Optional[TuneType] = None,
        base_checkpoint_id: typing.Optional[str] = None,
        trigger_words: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        engine: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListTunesResponse:
        """
        List all tunes owned by the current user.

        Parameters
        ----------
        offset : typing.Optional[int]
            Offset into the results.

        limit : typing.Optional[int]
            The max number of results to be shown (limit 100).

        name : typing.Optional[str]
            The name of the tune to filter on.

        tune_type : typing.Optional[TuneType]
            The type of the tune to filter on.

        base_checkpoint_id : typing.Optional[str]
            The base checkpoint ID for the LoRA tune.

        trigger_words : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            The trigger words to filter on.

        engine : typing.Optional[str]
            The engine type for listing tunes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListTunesResponse
            Successful Response

        Examples
        --------
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.fine_tuning.list()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/tunes",
            base_url=self._client_wrapper.get_environment().default,
            method="GET",
            params={
                "offset": offset,
                "limit": limit,
                "name": name,
                "tune_type": tune_type,
                "base_checkpoint_id": base_checkpoint_id,
                "trigger_words": trigger_words,
                "engine": engine,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ListTunesResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from .errors.unprocessable_entity_error import UnprocessableEntityError
from .types.asset_type import AssetType
from .types.base_engine import BaseEngine
from .types.base_engine_type import BaseEngineType
from .types.complete_asset_upload_response import CompleteAssetUploadResponse
from .types.create_asset_response import CreateAssetResponse
from .types.data import Data
from .types.data_type import DataType
from .types.delete_asset_response import DeleteAssetResponse
from .types.http_validation_error import HttpValidationError
from .types.list_assets_response import ListAssetsResponse
from .types.retrieve_asset_response import RetrieveAssetResponse
from .types.transfer_api_type import TransferApiType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AssetLibraryClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        name: typing.Optional[str] = None,
        is_public: typing.Optional[bool] = None,
        data_type: typing.Optional[DataType] = None,
        asset_type: typing.Optional[typing.Union[AssetType, typing.Sequence[AssetType]]] = None,
        engine: typing.Optional[typing.Union[BaseEngine, typing.Sequence[BaseEngine]]] = None,
        engine_type: typing.Optional[BaseEngineType] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        get_preview_urls: typing.Optional[bool] = None,
        asset_ids: typing.Optional[str] = None,
        owner: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListAssetsResponse:
        """
        List assets with optional filtering.

        Parameters
        ----------
        name : typing.Optional[str]
            Filter based on asset name.

        is_public : typing.Optional[bool]
            Filter based on asset visibility.

        data_type : typing.Optional[DataType]
            Filter based on data type.

        asset_type : typing.Optional[typing.Union[AssetType, typing.Sequence[AssetType]]]
            Filter based on engine type.

        engine : typing.Optional[typing.Union[BaseEngine, typing.Sequence[BaseEngine]]]
            Filter based on engine.

        engine_type : typing.Optional[BaseEngineType]
            Filter based on engine type.

        limit : typing.Optional[int]
            Max number of assets to return.

        offset : typing.Optional[int]
            Offset to start listing assets.

        get_preview_urls : typing.Optional[bool]
            Include preview URLs for assets.

        asset_ids : typing.Optional[str]
            Filter based on asset IDs.

        owner : typing.Optional[str]
            Filter based on asset owener.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListAssetsResponse
            Successful Response

        Examples
        --------
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.asset_library.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/assets",
            base_url=self._client_wrapper.get_environment().default,
            method="GET",
            params={
                "name": name,
                "is_public": is_public,
                "data_type": data_type,
                "asset_type": asset_type,
                "engine": engine,
                "engine_type": engine_type,
                "limit": limit,
                "offset": offset,
                "get_preview_urls": get_preview_urls,
                "asset_ids": asset_ids,
                "owner": owner,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ListAssetsResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        asset_type: AssetType,
        data: Data,
        name: str,
        description: typing.Optional[str] = OMIT,
        hf_repo: typing.Optional[str] = OMIT,
        hf_token_secret: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        skip_validation: typing.Optional[bool] = OMIT,
        transfer_api_type: typing.Optional[TransferApiType] = OMIT,
        url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateAssetResponse:
        """
        Create an asset on the asset library.

        Parameters
        ----------
        asset_type : AssetType

        data : Data
            Asset data. It shows the type of data for an asset.

        name : str
            Asset name. A unique name for the asset.

        description : typing.Optional[str]
            Asset description. It is used to provide more information.

        hf_repo : typing.Optional[str]
            HuggingFace repository.

        hf_token_secret : typing.Optional[str]
            OctoAI secret name for HuggingFace token.

        is_public : typing.Optional[bool]
            True if asset is public.

        skip_validation : typing.Optional[bool]
            Skip asset validation.

        transfer_api_type : typing.Optional[TransferApiType]
            Transfer API type.

        url : typing.Optional[str]
            Upload the contents of the url as the asset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateAssetResponse
            Successful Response

        Examples
        --------
        from octoai.asset_library import Data_Checkpoint
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.asset_library.create(
            asset_type="file",
            data=Data_Checkpoint(
                data_type="fp16",
                engine="text/llama-2-7b",
                file_format="safetensors",
            ),
            description="string",
            hf_repo="string",
            hf_token_secret="string",
            is_public=True,
            name="string",
            skip_validation=True,
            transfer_api_type="presigned-url",
            url="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/assets",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={
                "asset_type": asset_type,
                "data": data,
                "description": description,
                "hf_repo": hf_repo,
                "hf_token_secret": hf_token_secret,
                "is_public": is_public,
                "name": name,
                "skip_validation": skip_validation,
                "transfer_api_type": transfer_api_type,
                "url": url,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(CreateAssetResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeleteAssetResponse:
        """
        Delete an asset from the asset library.

        Parameters
        ----------
        asset_id : str
            The asset unique ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteAssetResponse
            Successful Response

        Examples
        --------
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.asset_library.delete(
            asset_id="asset_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/assets/{jsonable_encoder(asset_id)}",
            base_url=self._client_wrapper.get_environment().default,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(DeleteAssetResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def complete_upload(
        self,
        asset_id: str,
        *,
        skip_validation: typing.Optional[bool] = OMIT,
        token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CompleteAssetUploadResponse:
        """
        Complete the upload of an asset.

        Parameters
        ----------
        asset_id : str
            The asset unique ID.

        skip_validation : typing.Optional[bool]
            Skip asset validation.

        token : typing.Optional[str]
            Unused. User OctoAI token.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CompleteAssetUploadResponse
            Successful Response

        Examples
        --------
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.asset_library.complete_upload(
            asset_id="asset_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/assets/{jsonable_encoder(asset_id)}/complete-upload",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={"skip_validation": skip_validation, "token": token},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(CompleteAssetUploadResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        asset_owner_and_name_or_id: str,
        *,
        transfer_api_type: typing.Optional[TransferApiType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RetrieveAssetResponse:
        """
        Retrieve asset with the given owner and name or id.

        Parameters
        ----------
        asset_owner_and_name_or_id : str
            Owner and name or id of the asset.

        transfer_api_type : typing.Optional[TransferApiType]
            Type of transfer API to generate.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveAssetResponse
            Successful Response

        Examples
        --------
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.asset_library.get(
            asset_owner_and_name_or_id="string",
            transfer_api_type="presigned-url",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/assets/{jsonable_encoder(asset_owner_and_name_or_id)}",
            base_url=self._client_wrapper.get_environment().default,
            method="GET",
            params={"transfer_api_type": transfer_api_type},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(RetrieveAssetResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAssetLibraryClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        name: typing.Optional[str] = None,
        is_public: typing.Optional[bool] = None,
        data_type: typing.Optional[DataType] = None,
        asset_type: typing.Optional[typing.Union[AssetType, typing.Sequence[AssetType]]] = None,
        engine: typing.Optional[typing.Union[BaseEngine, typing.Sequence[BaseEngine]]] = None,
        engine_type: typing.Optional[BaseEngineType] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        get_preview_urls: typing.Optional[bool] = None,
        asset_ids: typing.Optional[str] = None,
        owner: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListAssetsResponse:
        """
        List assets with optional filtering.

        Parameters
        ----------
        name : typing.Optional[str]
            Filter based on asset name.

        is_public : typing.Optional[bool]
            Filter based on asset visibility.

        data_type : typing.Optional[DataType]
            Filter based on data type.

        asset_type : typing.Optional[typing.Union[AssetType, typing.Sequence[AssetType]]]
            Filter based on engine type.

        engine : typing.Optional[typing.Union[BaseEngine, typing.Sequence[BaseEngine]]]
            Filter based on engine.

        engine_type : typing.Optional[BaseEngineType]
            Filter based on engine type.

        limit : typing.Optional[int]
            Max number of assets to return.

        offset : typing.Optional[int]
            Offset to start listing assets.

        get_preview_urls : typing.Optional[bool]
            Include preview URLs for assets.

        asset_ids : typing.Optional[str]
            Filter based on asset IDs.

        owner : typing.Optional[str]
            Filter based on asset owener.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListAssetsResponse
            Successful Response

        Examples
        --------
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.asset_library.list()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/assets",
            base_url=self._client_wrapper.get_environment().default,
            method="GET",
            params={
                "name": name,
                "is_public": is_public,
                "data_type": data_type,
                "asset_type": asset_type,
                "engine": engine,
                "engine_type": engine_type,
                "limit": limit,
                "offset": offset,
                "get_preview_urls": get_preview_urls,
                "asset_ids": asset_ids,
                "owner": owner,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ListAssetsResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        asset_type: AssetType,
        data: Data,
        name: str,
        description: typing.Optional[str] = OMIT,
        hf_repo: typing.Optional[str] = OMIT,
        hf_token_secret: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        skip_validation: typing.Optional[bool] = OMIT,
        transfer_api_type: typing.Optional[TransferApiType] = OMIT,
        url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateAssetResponse:
        """
        Create an asset on the asset library.

        Parameters
        ----------
        asset_type : AssetType

        data : Data
            Asset data. It shows the type of data for an asset.

        name : str
            Asset name. A unique name for the asset.

        description : typing.Optional[str]
            Asset description. It is used to provide more information.

        hf_repo : typing.Optional[str]
            HuggingFace repository.

        hf_token_secret : typing.Optional[str]
            OctoAI secret name for HuggingFace token.

        is_public : typing.Optional[bool]
            True if asset is public.

        skip_validation : typing.Optional[bool]
            Skip asset validation.

        transfer_api_type : typing.Optional[TransferApiType]
            Transfer API type.

        url : typing.Optional[str]
            Upload the contents of the url as the asset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateAssetResponse
            Successful Response

        Examples
        --------
        from octoai.asset_library import Data_Checkpoint
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.asset_library.create(
            asset_type="file",
            data=Data_Checkpoint(
                data_type="fp16",
                engine="text/llama-2-7b",
                file_format="safetensors",
            ),
            description="string",
            hf_repo="string",
            hf_token_secret="string",
            is_public=True,
            name="string",
            skip_validation=True,
            transfer_api_type="presigned-url",
            url="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/assets",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={
                "asset_type": asset_type,
                "data": data,
                "description": description,
                "hf_repo": hf_repo,
                "hf_token_secret": hf_token_secret,
                "is_public": is_public,
                "name": name,
                "skip_validation": skip_validation,
                "transfer_api_type": transfer_api_type,
                "url": url,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(CreateAssetResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteAssetResponse:
        """
        Delete an asset from the asset library.

        Parameters
        ----------
        asset_id : str
            The asset unique ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteAssetResponse
            Successful Response

        Examples
        --------
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.asset_library.delete(
            asset_id="asset_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/assets/{jsonable_encoder(asset_id)}",
            base_url=self._client_wrapper.get_environment().default,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(DeleteAssetResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def complete_upload(
        self,
        asset_id: str,
        *,
        skip_validation: typing.Optional[bool] = OMIT,
        token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CompleteAssetUploadResponse:
        """
        Complete the upload of an asset.

        Parameters
        ----------
        asset_id : str
            The asset unique ID.

        skip_validation : typing.Optional[bool]
            Skip asset validation.

        token : typing.Optional[str]
            Unused. User OctoAI token.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CompleteAssetUploadResponse
            Successful Response

        Examples
        --------
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.asset_library.complete_upload(
            asset_id="asset_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/assets/{jsonable_encoder(asset_id)}/complete-upload",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={"skip_validation": skip_validation, "token": token},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(CompleteAssetUploadResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        asset_owner_and_name_or_id: str,
        *,
        transfer_api_type: typing.Optional[TransferApiType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RetrieveAssetResponse:
        """
        Retrieve asset with the given owner and name or id.

        Parameters
        ----------
        asset_owner_and_name_or_id : str
            Owner and name or id of the asset.

        transfer_api_type : typing.Optional[TransferApiType]
            Type of transfer API to generate.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveAssetResponse
            Successful Response

        Examples
        --------
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.asset_library.get(
            asset_owner_and_name_or_id="string",
            transfer_api_type="presigned-url",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/assets/{jsonable_encoder(asset_owner_and_name_or_id)}",
            base_url=self._client_wrapper.get_environment().default,
            method="GET",
            params={"transfer_api_type": transfer_api_type},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(RetrieveAssetResponse, _response.json())  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from .function_choice import FunctionChoice

CreateChatCompletionRequestToolChoice = typing.Union[typing.Any, FunctionChoice]
